mod Mxgr logic: 'cl'

incl Mtex

incl Mtexxgr

rem 
  \para \bf  Arithmetization of Recursive Terms and Recursive Function Symbols. 
   \end 

rem 
  \para \bf  Constructors of recursive terms.  \end 

fun Vt 'Tex_f1_xgr_vt'
  Vt(i) = 0,i

fun/0 Zt 'Tex_f0_xgr_zt'
  Zt = 1,0

fun/3 Dt 'Tex_f3_xgr_dt'
  Dt(t1,t2,t3) = 2,t1,t2,t3

fun/2 Ct 'Tex_f2_xgr_ct'
  Ct(t,ts) = 3,t,ts

fun/2 At 'Tex_f2_xgr_at'
  At(e,ts) = 4,e,ts

rem 
  \para \bf  Constructors of recursive function symbols.  \end 

fun/0 Sx 'Tex_f0_xgr_sx'
  Sx = 5,0

fun/0 Px 'Tex_f0_xgr_px'
  Px = 6,0

fun Fx 'Tex_f2_xgr_fx'
  Fx(n) = 7,n

fun/2 Lx 'Tex_f2_xgr_lx'
  Lx(n,t) = 8,n,t

rem 
  \para \bf  Contraction of arguments.  \end 

rem 
  \para \it  Projection functions.  \end 

fun Proj1 'Tex_f1_pp_proj1'
  Proj1 Ct(t,ts) = t

fun Proj2 'Tex_f1_pp_proj2'
  Proj2 Ct(t,ts) = ts

rem 
  \para \it  Unary iteration of the second projection.  \end 

fun/2 Iproj2 'Tex_f2_pp_iproj2'
  Iproj2(0,ts) = ts
  Iproj2(n+1,ts) = Iproj2(n,Proj2(ts))

rem 
  \para \it  Predicate holding of tuples.  \end 

pred/2 Tuple 
  Tuple(n,ts) <-> n = 0 & ts = 0 \/ n = 1 \/ n >= 2 & Iproj2(n-2,ts) != 0

rem 
  \para \it  Projection function for tuples.  \end 

fun/3 Proj 'Tex_f3_pp_proj'
  Proj(n,ts,i) = Proj1 Iproj2(i-1,ts) <- i != n
  Proj(n,ts,i) = Iproj2(n-1,ts) <- i = n

rem 
  \para \bf  Recursive terms.  \end The predicate \ft Ctm(t) \end holds if the 
  number \ft t \end is the code of a closed recursive term. 

pred/3 Tm 
  Tm(n,Vt(i),1) <- 1 <= i & i <= n
  Tm(n,Zt,1)
  Tm(n,Dt(t1,t2,t3),1) <- Tm(n,t1,1) & Tm(n,t2,1) & Tm(n,t3,1)
  Tm(n,Ct(t,ts),k+1) <- Tm(n,t,1) & Tm(n,ts,k)
  Tm(n,At(Sx,t),1) <- Tm(n,t,1)
  Tm(n,At(Px,t),1) <- Tm(n,t,1)
  Tm(n,At(Fx(n1),ts),1) <- 1 <= n1 & n1 = n & Tm(n,ts,n)
  Tm(n,At(Lx(m,t),ts),1) <- 1 <= m & Tm(m,t,1) & Tm(n,ts,m)

pred Ctm 
  Ctm(t) <-> Tm(0,t,1)

rem 
  \para \bf  Well-formed recursive indices.  \end \header* pred/2 Rf  \end The 
  binary predicate \ft Rf(n,e) \end holds if the number \ft e \end is the code 
  of some \ft n \end-ary recursive function symbol. We say that the number 
  \ft e \end is a \it  well-formed recursive index  \end. 

pred/2 Rf 
  Rf(n,Sx) <- n = 1
  Rf(n,Px) <- n = 1
  Rf(n,Lx(n1,t)) <- n >= 1 & n1 = n & Tm(n,t,1)

rem 
  \para \bf  [CL] Formats.  \end The predicate \ft Fr \end is the union format 
  and it can be used to control the display of the codes of recursive terms and 
  recursive function symbols in queries. 

pred Fr 
  Fr Vt(i) <- N(i)
  Fr(Zt)
  Fr Dt(t1,t2,t3) <- Fr(t1) & Fr(t2) & Fr(t3)
  Fr Ct(t,ts) <- Fr(t) & Fr(ts)
  Fr At(e,ts) <- Fr(e) & Fr(ts)
  Fr(Sx)
  Fr(Px)
  Fr Fx(n) <- N(n)
  Fr Lx(n,t) <- N(n) & Fr(t)

